searchNodes=[{"doc":"Methods for interacting with ArangoDB that aren't through Ecto. This allows for easy interaction with graph functions of Arango. Using Ecto's relations for edge relations was tried but it was found to be too much of a 'hacky' solution. Using separate functions that still utilise Ecto document queries was found to be the optimal solution.","ref":"ArangoXEcto.html","title":"ArangoXEcto","type":"module"},{"doc":"Runs an Arangox function using a repo This is simply a helper function that extracts the connection from the repo and runs a regular query. Parameters repo - The Ecto repo module used for connection function - An atom of the Arangox function to run args - The options passed to the function (not including the conn argument) The conn argument is automatically prepended to your supplied args Supported Functions :abort :cursor :delete :delete! :get :get! :head :head! :options :options! :patch :patch! :post :post! :put :put! :request :request! :run :status :transaction (use built in Ecto.Repo.transaction/2 instead) Examples iex&gt; ArangoXEcto . api_query ( Repo , :get , [ &quot;/_api/collection&quot; ] ) { :ok , % Arangox.Response { body : ... } } iex&gt; ArangoXEcto . api_query ( Repo , :non_existent , [ &quot;/_api/collection&quot; ] ) ** (ArgumentError) Invalid function passed to `Arangox` module","ref":"ArangoXEcto.html#api_query/3","title":"ArangoXEcto.api_query/3","type":"function"},{"doc":"Same as ArangoXEcto.aql_query/4 but will raise an error. If there is an error in the query such as a syntax error, an Arangox.Error will be raised. Examples iex&gt; ArangoXEcto . aql_query! ( Repo , &quot;FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var&quot; , fname : &quot;John&quot; , lname : &quot;Smith&quot; ) [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot; , &quot;_key&quot; =&gt; &quot;12345&quot; , &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot; , &quot;first_name&quot; =&gt; &quot;John&quot; , &quot;last_name&quot; =&gt; &quot;Smith&quot; } ]","ref":"ArangoXEcto.html#aql_query!/4","title":"ArangoXEcto.aql_query!/4","type":"function"},{"doc":"Runs a raw AQL query on the database. This will create a transaction and cursor on Arango and run the raw query. If there is an error in the query such as a syntax error, an Arangox.Error will be raised. Parameters repo - The Ecto repo module to use for queries query - The AQL query string to execute vars - A keyword list or a map with the values for variables in the query opts - Options to be passed for the transaction Accepts any of the options accepted by DBConnection.transaction/3, as well as any of the following: :read - An array of collection names or a single collection name as a binary. :write - An array of collection names or a single collection name as a binary. :exclusive - An array of collection names or a single collection name as a binary. :properties - A list or map of additional body attributes to append to the request body when beginning a transaction. If doing a write operation, the :write operation must be passed. This is not explicitly required for read operations. Examples iex&gt; ArangoXEcto . aql_query ( Repo , &quot;FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var&quot; , fname : &quot;John&quot; , lname : &quot;Smith&quot; ) { :ok , [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot; , &quot;_key&quot; =&gt; &quot;12345&quot; , &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot; , &quot;first_name&quot; =&gt; &quot;John&quot; , &quot;last_name&quot; =&gt; &quot;Smith&quot; } ] }","ref":"ArangoXEcto.html#aql_query/4","title":"ArangoXEcto.aql_query/4","type":"function"},{"doc":"Checks if a collection exists This will return true if the collection exists in the database, matches the specified type and is not a system database, otherwise it will be false. Parameters repo - The Ecto repo module to use for the query collection_name - Name of the collection to check type - The type of collection to check against, defaults to a regular document opts - Options to be used, available options shown below, defaults to none Options :prefix - The prefix to use for database collection checking Examples Checking a document collection exists iex&gt; ArangoXEcto . collection_exists? ( Repo , :users ) true Checking an edge collection exists iex&gt; ArangoXEcto . collection_exists? ( Repo , &quot;my_edge&quot; , :edge ) true Checking a system document collection exists does not work iex&gt; ArangoXEcto . collection_exists? ( Repo , &quot;_system_test&quot; ) false Checking a collection exists with database prefix iex&gt; ArangoXEcto . collection_exists? ( Repo , &quot;test&quot; , &quot;tenant1&quot; ) true","ref":"ArangoXEcto.html#collection_exists?/4","title":"ArangoXEcto.collection_exists?/4","type":"function"},{"doc":"Creates analyzers in a module This will first force delete any analyzers with the same name before creating the ones defined. In production you should be using migrations in static mode. This function will be called when an analyzer module is passed to the ArangoXEcto.Migration.create/2 function. When operating in dynamic mode the analyzers ae automatically created on view create, see ArangoXEcto.create_view/2 for more info. Parameters repo - The Ecto repo module to use for queries analyzer_module - The Analyzer module opts - Additional options to use for analyzer creation Options :prefix - The prefix for the tenant to create analyzers for Examples iex&gt; ArangoXEcto . create_analyzers ( Repo , MyApp.Analyzers ) { :ok , [ % Arangox.Response { } , ... ] } If there is an error in and of the schemas iex&gt; ArangoXEcto . create_analyzers ( Repo , MyApp.Analyzers ) { :error , [ % Arangox.Response { } , ... ] , [ % Arangox.Error { } , ... ] }","ref":"ArangoXEcto.html#create_analyzers/3","title":"ArangoXEcto.create_analyzers/3","type":"function"},{"doc":"Creates a collection defined by some schema This function is only to be used in dynamic mode and will raise an error if called in static mode. Instead use migrations if in static mode. Parameters repo - The Ecto repo module to use for queries schema - The Collection Schema to be created opts - Additional options to pass Options :prefix - The prefix to use for database tenant collection creation Examples iex&gt; ArangoXEcto . create_collection ( Repo , MyApp.Users ) :ok If there is an error in the schema iex&gt; ArangoXEcto . create_collection ( Repo , MyApp.Users ) { :error , % Arangox.Error { } }","ref":"ArangoXEcto.html#create_collection/3","title":"ArangoXEcto.create_collection/3","type":"function"},{"doc":"Creates an edge between two modules This can create an edge collection dynamically if no additional fields are required, otherwise an edge schema needs to be specified. The collection name can be passed as an option or is obtained from the provided schema, otherwise it is generated dynamically. Since ArangoDB does not care about the order of the from and two options in anonymous graphs, the order of the from and to attributes used in this function will work either way. Parameters repo - The Ecto repo module to use for queries from - The Ecto Schema struct to use for the from vertex to - The Ecto Schema struct to use for the to vertex opts - Options to use Options Accepts the following options: :edge - A specific edge module to use for the edge. This is required for any additional fields on the edge. Overrides collection_name . :fields - The values of the fields to set on the edge. Requires edge to be set otherwise it is ignored. :collection_name - The name of the collection to use. Examples iex&gt; ArangoXEcto . create_edge ( Repo , user1 , user2 ) % UserUser { _from : &quot;users/12345&quot; , _to : &quot;users/54321&quot; } Create an edge with a specific edge collection name iex&gt; ArangoXEcto . create_edge ( Repo , user1 , user2 , collection_name : &quot;friends&quot; ) % Friends { _from : &quot;users/12345&quot; , _to : &quot;users/54321&quot; } Create a edge schema and use it to create an edge relation defmodule UserPosts do use ArangoXEcto.Edge , from : User , to : Post import Ecto.Changeset schema &quot;user_posts&quot; do edge_fields ( ) field ( :type , :string ) end def changeset ( edge , attrs ) do edges_changeset ( edge , attrs ) |&gt; cast ( attrs , [ :type ] ) |&gt; validate_required ( [ :type ] ) end end iex&gt; ArangoXEcto . create_edge ( Repo , user1 , user2 , edge : UserPosts , fields : %{ type : &quot;wrote&quot; } ) % UserPosts { _from : &quot;users/12345&quot; , _to : &quot;users/54321&quot; , from : # Ecto.Association.NotLoaded &lt; association :from is not loaded &gt; , to : # Ecto.Association.NotLoaded &lt; association :to is not loaded &gt; , type : &quot;wrote&quot; }","ref":"ArangoXEcto.html#create_edge/4","title":"ArangoXEcto.create_edge/4","type":"function"},{"doc":"Creates a view defined by some view schema When in dynamic mode this will automatically create analyzers and linked collections. If in static mode these need to be done seperately in the migrations. For the analyzers to be automatically created in dynamic mode the analyzers module needs to be passed as an option into the view module definition. See ArangoXEcto.View for more info. Parameters repo - The Ecto repo module to use for queries view - The View Schema to be created opts - Additional options to use for analyzer creation Options :prefix - The prefix for the tenant to create analyzers for Examples iex&gt; ArangoXEcto . create_view ( Repo , MyApp.Views.UserSearch ) { :ok , % Arangox.Response { } } If there is an error in the schema iex&gt; ArangoXEcto . create_view ( Repo , MyApp.Views.UserSearch ) { :error , % Arangox.Error { } }","ref":"ArangoXEcto.html#create_view/3","title":"ArangoXEcto.create_view/3","type":"function"},{"doc":"Deletes all edges matching matching the query If the :conditions option is set then those conditions must be true to delete. To just delete one edge do so like any other Ecto Schema struct, i.e. using Ecto.Repo methods. Parameters repo - The Ecto repo module to use for queries from - The Ecto Schema struct to use for the from vertex to - The Ecto Schema struct to use for the to vertex opts - Options to use Options Accepts the following options: :edge - A specific edge module to use for the edge. This is required for any additional fields on the edge. Overrides :collection_name . :collection_name - The name of the collection to use. :conditions - A keyword list of conditions to filter for edge deletion Examples Deletes all edges from user1 to user2 iex&gt; ArangoXEcto . delete_all_edges ( Repo , user1 , user2 ) :ok Deletes all edges from user1 to user2 in specific collection iex&gt; ArangoXEcto . delete_all_edges ( Repo , user1 , user2 , collection_name : &quot;friends&quot; ) :ok Deletes all edges from user1 to user2 that have matching conditions iex&gt; ArangoXEcto . delete_all_edges ( Repo , user1 , user2 , conditions : [ type : &quot;best_friend&quot; ] ) :ok","ref":"ArangoXEcto.html#delete_all_edges/4","title":"ArangoXEcto.delete_all_edges/4","type":"function"},{"doc":"Generates a edge schema dynamically If a collection name is not provided one will be dynamically generated. The naming convention is the names of the two modules is alphabetical order. E.g. User and Post will combine for a collection name of post_user and an edge module name of PostUser . This order is used to prevent duplicates if the from and to orders are switched. This will create the Ecto Module in the environment dynamically. It will create it under the closest common parent module of the passed modules plus the Edges alias. For example, if the modules were MyApp.Apple.User and MyApp.Apple.Banana.Post then the edge would be created at MyApp.Apple.Edges.PostUser . Returns the Edge Module name as an atom. Parameters from_module - Ecto Schema Module for the from part of the edge to_module - Ecto Schema Module for the to part of the edge opts - Options passed for module generation Options :collection_name - The name of collection to use instead of generating it Examples iex&gt; ArangoXEcto . edge_module ( MyProject.User , MyProject.Company , [ collection_name : &quot;works_for&quot; ] ) MyProject.WorksFor iex&gt; ArangoXEcto . edge_module ( MyProject.User , MyProject.Company ) MyProject.UsersCompanies","ref":"ArangoXEcto.html#edge_module/3","title":"ArangoXEcto.edge_module/3","type":"function"},{"doc":"Gets an ID from a module and a key Parameters module - Module to get the collection name from key - The _key to use in the id Example iex&gt; ArangoXEcto . get_id_from_module ( User , &quot;123456&quot; ) &quot;users/123456&quot;","ref":"ArangoXEcto.html#get_id_from_module/2","title":"ArangoXEcto.get_id_from_module/2","type":"function"},{"doc":"Gets an ID from a schema struct Parameters struct - The Ecto struct Example If the User schema's collection name is users the following would be: iex&gt; user = % User { id : &quot;123456&quot; } % User { id : &quot;123456&quot; } iex&gt; ArangoXEcto . get_id_from_struct ( user ) &quot;users/123456&quot;","ref":"ArangoXEcto.html#get_id_from_struct/1","title":"ArangoXEcto.get_id_from_struct/1","type":"function"},{"doc":"Returns the name of a database for prefix This simply prepends the tenant name to the database supplied in the repo config. Parameters repo - The Ecto repo module to use for the query prefix - The prefix to get the database name for","ref":"ArangoXEcto.html#get_prefix_database/2","title":"ArangoXEcto.get_prefix_database/2","type":"function"},{"doc":"Returns if a Schema is a document schema or not Checks for the presence of the __schema__/1 function on the module and not an edge.","ref":"ArangoXEcto.html#is_document?/1","title":"ArangoXEcto.is_document?/1","type":"function"},{"doc":"Returns if a Schema is an edge or not Checks for the presence of the __edge__/0 function on the module.","ref":"ArangoXEcto.html#is_edge?/1","title":"ArangoXEcto.is_edge?/1","type":"function"},{"doc":"Returns if a Schema is a view schema or not Checks for the presence of the __view__/1 function on the module.","ref":"ArangoXEcto.html#is_view?/1","title":"ArangoXEcto.is_view?/1","type":"function"},{"doc":"Loads raw map into ecto structs Uses Ecto.Repo.load/2 to load a deep map result into ecto structs If a list of maps are passed then the maps are enumerated over. A list of modules can also be passes for possible types. The _id field is used to check against the module schemas. This is especially useful for querying against arango views where there may be multiple schema results returned. If a module is passed that isn't a Ecto Schema then an error will be raised. Parameters maps - List of maps or singular map to convert to a struct module - Module(s) to use for the struct Example iex&gt; { :ok , users } = ArangoXEcto . aql_query ( Repo , &quot;FOR user IN users RETURN user&quot; ) { :ok , [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot; , &quot;_key&quot; =&gt; &quot;12345&quot; , &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot; , &quot;first_name&quot; =&gt; &quot;John&quot; , &quot;last_name&quot; =&gt; &quot;Smith&quot; } ] } iex&gt; ArangoXEcto . load ( users , User ) [ % User { id : &quot;12345&quot; , first_name : &quot;John&quot; , last_name : &quot;Smith&quot; } ]","ref":"ArangoXEcto.html#load/2","title":"ArangoXEcto.load/2","type":"function"},{"doc":"Converts raw output of a query into a struct Transforms string map arguments into atom key map, adds id key and drops _id , _key and _rev keys. Then it creates a struct from filtered arguments using module. If a list of maps are passed then the maps are enumerated over. Parameters maps - List of maps or singular map to convert to a struct module - Module to use for the struct Example iex&gt; { :ok , users } = ArangoXEcto . aql_query ( Repo , &quot;FOR user IN users RETURN user&quot; ) { :ok , [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot; , &quot;_key&quot; =&gt; &quot;12345&quot; , &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot; , &quot;first_name&quot; =&gt; &quot;John&quot; , &quot;last_name&quot; =&gt; &quot;Smith&quot; } ] } iex&gt; ArangoXEcto . raw_to_struct ( users , User ) [ % User { id : &quot;12345&quot; , first_name : &quot;John&quot; , last_name : &quot;Smith&quot; } ]","ref":"ArangoXEcto.html#raw_to_struct/2","title":"ArangoXEcto.raw_to_struct/2","type":"function"},{"doc":"Same as schema_type/1 but throws an error on none This is just a shortcut to using is_edge/1 , is_document/1 and is_view/1 . If it is none of them nil is returned. Examples A real edge schema iex&gt; ArangoXEcto . schema_type! ( MyApp.RealEdge ) :edge Some module that is not an Ecto schema iex&gt; ArangoXEcto . schema_type! ( MyApp.RandomModule ) ** (ArgumentError) Not an Ecto Schema","ref":"ArangoXEcto.html#schema_type!/1","title":"ArangoXEcto.schema_type!/1","type":"function"},{"doc":"Returns the type of a module This is just a shortcut to using is_edge/1 , is_document/1 and is_view/1 . If it is none of them nil is returned. Examples A real view iex&gt; ArangoXEcto . schema_type ( MyApp.SomeView ) :view A real edge schema iex&gt; ArangoXEcto . schema_type ( MyApp.RealEdge ) :edge Some module that is not an Ecto schema iex&gt; ArangoXEcto . schema_type ( MyApp.RandomModule ) nil","ref":"ArangoXEcto.html#schema_type/1","title":"ArangoXEcto.schema_type/1","type":"function"},{"doc":"Checks if a view exists This will return true if the view exists in the database, otherwise false. Parameters repo - The Ecto repo module to use for the query view_name - Name of the collection to check Examples Checking a document collection exists iex&gt; ArangoXEcto . view_exists? ( Repo , :users_search ) true","ref":"ArangoXEcto.html#view_exists?/2","title":"ArangoXEcto.view_exists?/2","type":"function"},{"doc":"","ref":"ArangoXEcto.html#t:mod/0","title":"ArangoXEcto.mod/0","type":"type"},{"doc":"","ref":"ArangoXEcto.html#t:query/0","title":"ArangoXEcto.query/0","type":"type"},{"doc":"","ref":"ArangoXEcto.html#t:vars/0","title":"ArangoXEcto.vars/0","type":"type"},{"doc":"Ecto adapter for ArangoDB using ArangoX This implements methods for Ecto.Adapter . These functions should not be accessed directly and should only be used by Ecto. Direct interaction functions are in the ArangoXEcto module.","ref":"ArangoXEcto.Adapter.html","title":"ArangoXEcto.Adapter","type":"module"},{"doc":"Returns true if a connection has been checked out","ref":"ArangoXEcto.Adapter.html#checked_out?/1","title":"ArangoXEcto.Adapter.checked_out?/1","type":"function"},{"doc":"Checks out a connection for the duration of the given function.","ref":"ArangoXEcto.Adapter.html#checkout/3","title":"ArangoXEcto.Adapter.checkout/3","type":"function"},{"doc":"Returns the dumpers for a given type.","ref":"ArangoXEcto.Adapter.html#dumpers/2","title":"ArangoXEcto.Adapter.dumpers/2","type":"function"},{"doc":"Ensure all applications necessary to run the adapter are started","ref":"ArangoXEcto.Adapter.html#ensure_all_started/2","title":"ArangoXEcto.Adapter.ensure_all_started/2","type":"function"},{"doc":"Initialise adapter with config","ref":"ArangoXEcto.Adapter.html#init/1","title":"ArangoXEcto.Adapter.init/1","type":"function"},{"doc":"","ref":"ArangoXEcto.Adapter.html#load_decimal/1","title":"ArangoXEcto.Adapter.load_decimal/1","type":"function"},{"doc":"","ref":"ArangoXEcto.Adapter.html#load_float/1","title":"ArangoXEcto.Adapter.load_float/1","type":"function"},{"doc":"","ref":"ArangoXEcto.Adapter.html#load_integer/1","title":"ArangoXEcto.Adapter.load_integer/1","type":"function"},{"doc":"Returns the loaders of a given type.","ref":"ArangoXEcto.Adapter.html#loaders/2","title":"ArangoXEcto.Adapter.loaders/2","type":"function"},{"doc":"Starts the Agent with an empty list","ref":"ArangoXEcto.Adapter.html#start_link/1","title":"ArangoXEcto.Adapter.start_link/1","type":"function"},{"doc":"Defines an analyzer for use in views Since analyzer defintions are short and you may have many of them, you can just define multiple analyzer modules in one file, e.g. named analyzers.ex. Example defmodule MyApp.Analyzers do use ArangoXEcto.Analyzer norm :norm_en , [ :frequency , :norm , :position ] , %{ locale : &quot;en&quot; , accent : false , case : :lower } # this exists by default but is just used as an example text :text_en , [ :frequency , :norm , :position ] , %{ locale : &quot;en&quot; , accent : false , stemming : true , case : :lower } # Needed to compile the analyzers build ( ) end Features The following are the features available to all the analyzers. Some analyzers and functions need certin features enabled, refer to the ArangoDB docs for more info. * ` :frequency ` - ( boolean ) - track how often a term occurs . * ` :norm ` - ( boolean ) - calculate and store the field normalization factor that is used to score fairer if the same term is repeated , reducing its importance . * ` :position ` - ( boolean ) - enumerate the tokens for position - dependent queries .","ref":"ArangoXEcto.Analyzer.html","title":"ArangoXEcto.Analyzer","type":"module"},{"doc":"Defines an aql type analyzer. An Analyzer capable of running a restricted AQL query to perform data manipulation / filtering. Refer to the ArangoDB AQL Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` queryString ` - ( string ) - AQL query to be executed . * ` collapsePositions ` - ( boolean ) - whether to set the position to 0 for all members of the query result array ( true ) or set the position corresponding to the index of the result array member ( false ) . * ` keepNull ` - ( boolean ) - either treat treat null like an empty string or discard null . * ` batchSize ` - ( integer ) - number between 1 and 1000 ( default = 1 ) that determines the batch size for reading data from the query . * ` memoryLimit ` - ( integer ) - memory limit for query execution in bytes . ( default is 1048576 = 1 Mb ) Maximum is 33554432 U ( 32 Mb ) . * ` returnType ` - ( string ) - data type of the returned tokens . ` :string ` - convert emitted tokens to strings . ` :number ` - convert emitted tokens to numbers . ` :bool ` - convert emitted tokens to booleans .","ref":"ArangoXEcto.Analyzer.html#aql/3","title":"ArangoXEcto.Analyzer.aql/3","type":"macro"},{"doc":"Compiles analyzers","ref":"ArangoXEcto.Analyzer.html#build/0","title":"ArangoXEcto.Analyzer.build/0","type":"macro"},{"doc":"Defines a classification type analyzer. An Analyzer capable of classifying tokens in the input text. Refer to the ArangoDB Classification Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` model_location ` - ( string ) - the on - disk path to the trained fastText supervised model . * ` top_k ` - ( number ) - the number of class labels that will be produced per input ( default : 1 ) . * ` threshold ` - ( number ) - the probability threshold for which a label will be assigned to an input . A fastText model produces a probability per class label , and this is what will be filtered ( default : 0.99 ) .","ref":"ArangoXEcto.Analyzer.html#classification/3","title":"ArangoXEcto.Analyzer.classification/3","type":"macro"},{"doc":"Defines a collation type analyzer. An Analyzer capable of converting the input into a set of language-specific tokens. This makes comparisons follow the rules of the respective language, most notable in range queries against Views. Refer to the ArangoDB Collation Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` locale ` - ( string ) - a locale in the format language [ _COUNTRY ] ( square brackets denote optional parts ) , e . g . &quot;de&quot; or &quot;en_US&quot; .","ref":"ArangoXEcto.Analyzer.html#collation/3","title":"ArangoXEcto.Analyzer.collation/3","type":"macro"},{"doc":"Defines a delimiter type analyzer. An Analyzer capable of breaking up delimited text into tokens as per RFC 4180 (without starting new records on newlines). Refer to the ArangoDB Delimiter Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` delimiter ` - ( string ) - the delimiting character ( s ) . The whole string is considered as one delimiter .","ref":"ArangoXEcto.Analyzer.html#delimiter/3","title":"ArangoXEcto.Analyzer.delimiter/3","type":"macro"},{"doc":"Defines a geo_s2 type analyzer. An Analyzer capable of breaking up a GeoJSON object or coordinate array in [longitude, latitude] order into a set of indexable tokens for further usage with ArangoSearch Geo functions. Refer to the ArangoDB Geo S2 Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` format ` - ( atom ) - the internal binary representation to use for storing the geo - spatial data in an index * ` :latLngDouble ` ( default ) - store each latitude and longitude value as an 8 - byte floating - point value ( 16 bytes per coordinate pair ) . This format preserves numeric values exactly and is more compact than the VelocyPack format used by the geojson Analyzer . * ` :latLngInt ` - store each latitude and longitude value as an 4 - byte integer value ( 8 bytes per coordinate pair ) . This is the most compact format but the precision is limited to approximately 1 to 10 centimeters . * ` :s2Point ` - store each longitude - latitude pair in the native format of Google S2 which is used for geo - spatial calculations ( 24 bytes per coordinate pair ) . This is not a particular compact format but it reduces the number of computations necessary when you execute geo - spatial queries . This format preserves numeric values exactly . * ` type ` - ( atom ) - type of geojson object * ` :shape ` ( default ) - index all GeoJSON geometry types ( Point , Polygon etc . ) * ` :centroid ` - compute and only index the centroid of the input geometry * ` :point ` - only index GeoJSON objects of type Point , ignore all other geometry types * ` options ` - ( map ) - options for fine - tuning geo queries . These options should generally remain unchanged * ` :maxCells ` ( number , optional ) - maximum number of S2 cells ( default : 20 ) * ` :minLevel ` ( number , optional ) - the least precise S2 level ( default : 4 ) * ` :maxLevel ` ( number , optional ) - the most precise S2 level ( default : 23 )","ref":"ArangoXEcto.Analyzer.html#geo_s2/3","title":"ArangoXEcto.Analyzer.geo_s2/3","type":"macro"},{"doc":"Defines a geojson type analyzer. An Analyzer capable of breaking up a GeoJSON object or coordinate array in [longitude, latitude] order into a set of indexable tokens for further usage with ArangoSearch Geo functions. Refer to the ArangoDB GeoJSON Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` type ` - ( atom ) - type of geojson object * ` :shape ` ( default ) - index all GeoJSON geometry types ( Point , Polygon etc . ) * ` :centroid ` - compute and only index the centroid of the input geometry * ` :point ` - only index GeoJSON objects of type Point , ignore all other geometry types * ` options ` - ( map ) - options for fine - tuning geo queries . These options should generally remain unchanged * ` :maxCells ` ( number , optional ) - maximum number of S2 cells ( default : 20 ) * ` :minLevel ` ( number , optional ) - the least precise S2 level ( default : 4 ) * ` :maxLevel ` ( number , optional ) - the most precise S2 level ( default : 23 )","ref":"ArangoXEcto.Analyzer.html#geojson/3","title":"ArangoXEcto.Analyzer.geojson/3","type":"macro"},{"doc":"Defines a geopoint type analyzer. An Analyzer capable of breaking up a coordinate array in [latitude, longitude] order or a JSON object describing a coordinate pair using two separate attributes into a set of indexable tokens for further usage with ArangoSearch Geo functions. Refer to the ArangoDB Geo Point Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` latitude ` - ( list of string ) - list of strings that describes the attribute path of the latitude value relative to the field for which the Analyzer is defined in the View * ` longitude ` - ( list of string ) - list of strings that describes the attribute path of the longitude value relative to the field for which the Analyzer is defined in the View * ` options ` - ( map ) - options for fine - tuning geo queries . These options should generally remain unchanged * ` :maxCells ` ( number , optional ) - maximum number of S2 cells ( default : 20 ) * ` :minLevel ` ( number , optional ) - the least precise S2 level ( default : 4 ) * ` :maxLevel ` ( number , optional ) - the most precise S2 level ( default : 23 )","ref":"ArangoXEcto.Analyzer.html#geopoint/3","title":"ArangoXEcto.Analyzer.geopoint/3","type":"macro"},{"doc":"Defines an identity type analyzer. An Analyzer applying the identity transformation, i.e. returning the input unmodified. Refer to the ArangoDB Identity Docs for more info. This does not accept any properties. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features))","ref":"ArangoXEcto.Analyzer.html#identity/2","title":"ArangoXEcto.Analyzer.identity/2","type":"macro"},{"doc":"Defines a minhash type analyzer. An Analyzer that computes so called MinHash signatures using a locality-sensitive hash function. It applies an Analyzer of your choice before the hashing, for example, to break up text into words. Refer to the ArangoDB MinHash Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below * ` analyzer ` - block with one analyzer ( if more then one is supplied , only the last will be used Properties * ` numHashes ` - ( number ) - the size of the MinHash signature .","ref":"ArangoXEcto.Analyzer.html#minhash/4","title":"ArangoXEcto.Analyzer.minhash/4","type":"macro"},{"doc":"Defines a nearest_neighbors type analyzer. An Analyzer capable of finding nearest neighbors of tokens in the input. Refer to the ArangoDB Nearest Neighbors Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` model_location ` - ( string ) - the on - disk path to the trained fastText supervised model . * ` top_k ` - ( number ) - the number of class labels that will be produced per input ( default : 1 ) .","ref":"ArangoXEcto.Analyzer.html#nearest_neighbors/3","title":"ArangoXEcto.Analyzer.nearest_neighbors/3","type":"macro"},{"doc":"Defines a ngram type analyzer. An Analyzer capable of producing n-grams from a specified input in a range of min..max (inclusive). Can optionally preserve the original input. Refer to the ArangoDB NGram Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` min ` - ( integer ) - minimum n - gram length . * ` max ` - ( integer ) - maximum n - gram length . * ` preserveOriginal ` - ( boolean ) - whether to include the original value or just use the min &amp; max values . * ` startMarker ` - ( string ) - this value will be prepended to n - grams which include the beginning of the input . * ` endMarker ` - ( string ) - this value will be appended to n - grams which include the end of the input . * ` streamType ` - ( atom ) - type of the input stream . * ` :binary ` - one byte is considered as one character ( default ) . * ` :utf8 ` - one Unicode codepoint is treated as one character .","ref":"ArangoXEcto.Analyzer.html#ngram/3","title":"ArangoXEcto.Analyzer.ngram/3","type":"macro"},{"doc":"Defines a norm type analyzer. An Analyzer capable of normalizing the text, treated as a single token, i.e. case conversion and accent removal. Refer to the ArangoDB Norm Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` locale ` - ( string ) - a locale in the format language [ _COUNTRY ] ( square brackets denote optional parts ) , e . g . &quot;de&quot; or &quot;en_US&quot; . * ` accent ` - ( boolean ) - whether to preserve accented characters or convert them to the base characters . * ` case ` - ( atom ) - option of how to store case * ` :lower ` - to convert to all lower - case characters * ` :upper ` - to convert to all upper - case characters * ` :none ` - to not change character case ( default )","ref":"ArangoXEcto.Analyzer.html#norm/3","title":"ArangoXEcto.Analyzer.norm/3","type":"macro"},{"doc":"Defines a pipeline type analyzer. An Analyzer capable of chaining effects of multiple Analyzers into one. The pipeline is a list of Analyzers, where the output of an Analyzer is passed to the next for further processing. The final token value is determined by last Analyzer in the pipeline. Refer to the ArangoDB Pipeline Docs for more info. Note Features are only required on the pipeline and not on the individual analyzers within. Any features on sub analyzers will be ignored if supplied. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` analyzers ` - a block with other analyzers Example pipeline :my_pipeline , [ :frequency , :norm , :position ] do norm &quot;norm_en&quot; , %{ locale : &quot;en&quot; , accent : false , case : :lower } text &quot;text_en&quot; , %{ locale : &quot;en&quot; , accent : false , stemming : true , case : :lower } end","ref":"ArangoXEcto.Analyzer.html#pipeline/3","title":"ArangoXEcto.Analyzer.pipeline/3","type":"macro"},{"doc":"Defines a segmentation type analyzer. An Analyzer capable of breaking up the input text into tokens in a language-agnostic manner as per Unicode Standard Annex #29, making it suitable for mixed language strings. Refer to the ArangoDB Segmentation Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` break ` - ( atom ) - character to break at * ` :all ` - return all tokens * ` :alpha ` - return tokens composed of alphanumeric characters only ( default ) . Alphanumeric characters are Unicode codepoints from the Letter and Number categories , see Unicode Technical Note #36. * ` :graphic ` - return tokens composed of non - whitespace characters only . Note that the list of whitespace characters does not include line breaks : * ` U + 0009 ` Character Tabulation * ` U + 0020 ` Space * ` U + 0085 ` Next Line * ` U + 00 A0 ` No - break Space * ` U + 1680 ` Ogham Space Mark * ` U + 2000 ` En Quad * ` U + 2028 ` Line Separator * ` U + 202 F ` Narrow No - break Space * ` U + 205 F ` Medium Mathematical Space * ` U + 3000 ` Ideographic Space * ` case ` - ( atom ) - option of how to store case * ` :lower ` - to convert to all lower - case characters * ` :upper ` - to convert to all upper - case characters * ` :none ` - to not change character case ( default )","ref":"ArangoXEcto.Analyzer.html#segmentation/3","title":"ArangoXEcto.Analyzer.segmentation/3","type":"macro"},{"doc":"Defines a stem type analyzer. An Analyzer capable of stemming the text, treated as a single token, for supported languages. Refer to the ArangoDB Stem Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` locale ` - ( string ) - a locale in the format language [ _COUNTRY ] ( square brackets denote optional parts ) , e . g . &quot;de&quot; or &quot;en_US&quot; .","ref":"ArangoXEcto.Analyzer.html#stem/3","title":"ArangoXEcto.Analyzer.stem/3","type":"macro"},{"doc":"Defines a stopwords type analyzer. An Analyzer capable of removing specified tokens from the input. Refer to the ArangoDB Stopwords Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` stopwords ` - ( list of strings ) - array of strings that describe the tokens to be discarded . * ` hex ` - ( boolean ) - If false ( default ) , then each string in stopwords is used verbatim .","ref":"ArangoXEcto.Analyzer.html#stopwords/3","title":"ArangoXEcto.Analyzer.stopwords/3","type":"macro"},{"doc":"Defines a text type analyzer. An Analyzer capable of breaking up strings into individual words while also optionally filtering out stop-words, extracting word stems, applying case conversion and accent removal. Refer to the ArangoDB Text Docs for more info. Parameters * ` name ` - atom of the analyzer name * ` features ` - the features options to be set ( see [ Analyzer Features ] ( https :// hexdocs . pm / arangox_ecto / ArangoXEcto.Analyzer . html #module-features)) * ` properties ` - a map of the properties to be set , see below Properties * ` locale ` - ( string ) - a locale in the format language [ _COUNTRY ] ( square brackets denote optional parts ) , e . g . &quot;de&quot; or &quot;en_US&quot; . * ` accent ` - ( boolean ) - whether to preserve accented characters or convert them to the base characters . * ` case ` - ( string ) - option of how to store case * ` :lower ` - to convert to all lower - case characters * ` :upper ` - to convert to all upper - case characters * ` :none ` - to not change character case ( default ) * ` stemming ` - ( boolean ) - whether to apply stemming on returned words or leave as - is * ` edgeNgram ` - ( map ) - if present , then edge n - grams are generated for each token ( word ) . * ` min ` - ( integer ) - minimum n - gram length . * ` max ` - ( integer ) - maximum n - gram length . * ` preserveOriginal ` - ( boolean ) - whether to include the original value or just use the min &amp; max values . * ` stopwords ` - ( list of strings ) - a list of strings with words to omit from result . * ` stopwordsPath ` - ( string ) - path with a language sub - directory ( e . g . en for a locale en_US ) containing files with words to omit .","ref":"ArangoXEcto.Analyzer.html#text/3","title":"ArangoXEcto.Analyzer.text/3","type":"macro"},{"doc":"Defines an Arango Edge collection as an Ecto Schema Module. Edge modules are dynamically created in the environment if they don't already exist. For more on how edge modules are dynamically generated, please read ArangoXEcto.edge_module/3 . This will define the required fields of an edge ( _from and _to ) and will define the default changeset. Edges utilise Ecto relationships so that the powerful Ecto features can be used. Each Node requires a relationship to be set, either a outgoing/3 or incoming/3 . Behind the scenes this creates an Ecto many to many relationship and generates (or uses the provided) edge module as the intermediary schema. Since the edge collection uses the full _id instead of the _key for the _from and _to fields, once using any of the previously specified relationships a `__id field will be added to structs that will store the value of the _id` field so that relations can be loaded by Ecto. Extending Edge Schemas If you need to add additional fields to an edge, you can do so by creating your own edge module and defining the required fields as well as any additional fields. Luckily there are some helper macros so you don't have to do this manually again. When using a custom edge module, it must be passed to the relationship macros in the nodes using the :edge option, read more about these relationships at ArangoXEcto.Schema . Additionally, the :edge option must also be passed to the ArangoXEcto.create_edge/4 function. A custom schema module must use this module by adding use ArangoXEcto.Edge, from: FromSchema, to: ToSchema . When defining the fields in your schema, make sure to call edge_fields/1 . This will add the _from and _to foreign keys to the schema. It does not have to be before any custom fields but it good convention to do so. A changeset/2 function is automatically defined on the custom schema module but this must be overridden this so that you can cast and validate the custom fields. The edges_changeset/2 method should be called to automatically implement the casting and validation of the _from and _to fields. It does not have to be before any custom field operations but it good convention to do so. Example defmodule MyProject.UserPosts do use ArangoXEcto.Edge , from : User , to : Post import Ecto.Changeset schema &quot;user_posts&quot; do edge_fields ( ) field ( :type , :string ) end def changeset ( edge , attrs ) do edges_changeset ( edge , attrs ) |&gt; cast ( attrs , [ :type ] ) |&gt; validate_required ( [ :type ] ) end end","ref":"ArangoXEcto.Edge.html","title":"ArangoXEcto.Edge","type":"behaviour"},{"doc":"","ref":"ArangoXEcto.Edge.html#c:changeset/2","title":"ArangoXEcto.Edge.changeset/2","type":"callback"},{"doc":"Macro to define the required edge fields i.e. _from and _to . This is required when using a custom edge schema and can be used as below. schema &quot;user_posts&quot; do edge_fields ( ) field ( :type , :string ) end","ref":"ArangoXEcto.Edge.html#edge_fields/0","title":"ArangoXEcto.Edge.edge_fields/0","type":"macro"},{"doc":"Default changeset for an edge. Casts and requires the _from and _to fields. This will also verify the format of both fields to match that of an Arango id. Any custom changeset should first use this changeset. Direct use of the edges_changeset/2 function is discouraged unless per the use case mentioned above. Example To add a required type field, you could do the following: def changeset ( edge , attrs ) do edges_changeset ( edge , attrs ) |&gt; cast ( attrs , [ :type ] ) |&gt; validate_required ( [ :type ] ) end","ref":"ArangoXEcto.Edge.html#edges_changeset/2","title":"ArangoXEcto.Edge.edges_changeset/2","type":"function"},{"doc":"","ref":"ArangoXEcto.Edge.html#t:t/0","title":"ArangoXEcto.Edge.t/0","type":"type"},{"doc":"Methods for interacting with ArangoDB GeoJSON and geo related functions The methods within this module are really just helpers to generate Geo structs.","ref":"ArangoXEcto.GeoData.html","title":"ArangoXEcto.GeoData","type":"module"},{"doc":"","ref":"ArangoXEcto.GeoData.html#is_coordinate/1","title":"ArangoXEcto.GeoData.is_coordinate/1","type":"macro"},{"doc":"","ref":"ArangoXEcto.GeoData.html#is_latitude/1","title":"ArangoXEcto.GeoData.is_latitude/1","type":"macro"},{"doc":"","ref":"ArangoXEcto.GeoData.html#is_longitude/1","title":"ArangoXEcto.GeoData.is_longitude/1","type":"macro"},{"doc":"Generates a Geo linestring","ref":"ArangoXEcto.GeoData.html#linestring/1","title":"ArangoXEcto.GeoData.linestring/1","type":"function"},{"doc":"Generates a Geo multi linestring","ref":"ArangoXEcto.GeoData.html#multi_linestring/1","title":"ArangoXEcto.GeoData.multi_linestring/1","type":"function"},{"doc":"Generates a Geo multi point","ref":"ArangoXEcto.GeoData.html#multi_point/1","title":"ArangoXEcto.GeoData.multi_point/1","type":"function"},{"doc":"Generates a Geo multi polygon","ref":"ArangoXEcto.GeoData.html#multi_polygon/1","title":"ArangoXEcto.GeoData.multi_polygon/1","type":"function"},{"doc":"Generates a Geo point","ref":"ArangoXEcto.GeoData.html#point/2","title":"ArangoXEcto.GeoData.point/2","type":"function"},{"doc":"Generates a Geo polygon","ref":"ArangoXEcto.GeoData.html#polygon/1","title":"ArangoXEcto.GeoData.polygon/1","type":"function"},{"doc":"Sanitizes coordinates to ensure they are valid This function is not automatically applied to Geo constructors and must be applied before hand","ref":"ArangoXEcto.GeoData.html#sanitize/1","title":"ArangoXEcto.GeoData.sanitize/1","type":"function"},{"doc":"","ref":"ArangoXEcto.GeoData.html#t:coordinate/0","title":"ArangoXEcto.GeoData.coordinate/0","type":"type"},{"doc":"Defines Ecto Migrations for ArangoDB NOTE ArangoXEcto dynamically creates collections for you by default. Depending on your project architecture you may decide to use static migrations instead in which case this module will be useful. Migrations must use this module, otherwise migrations will not work. To do this, replace use Ecto.Migration with use ArangoXEcto.Migration . Since ArangoDB is schemaless, no fields need to be provided, only the collection name. First create a collection struct using the collection/3 function. Then pass the collection struct to the create/1 function. To create indexes it is a similar process using the index/3 function. Order matters!! Make sure you create collections before indexes and views, and analyzers before views if they are used. In general this is a good order to follow: Analyzers &gt; Collections &gt; Indexes &gt; Views To drop the collection on a migration down, do the same as creation except use the drop/1 function instead of the create/1 function. Indexes are automatically removed when the collection is removed and cannot be deleted using the drop/1 function. Example defmodule MyProject.Repo.Migrations.CreateUsers do use ArangoXEcto.Migration def up do create ( MyProject.Analyzers ) create ( collection ( :users ) ) create ( index ( &quot;users&quot; , [ :email ] ) ) create ( MyProject.UsersView ) end def down do drop ( collection ( :users ) ) end end","ref":"ArangoXEcto.Migration.html","title":"ArangoXEcto.Migration","type":"module"},{"doc":"Creates a collection struct Used to in functions that perform actions on the database. Accepts a collection type parameter that can either be :document or :edge , otherwise it will raise an error. The default option is :document . Options Accepts an options parameter as the third argument. For available keys please refer to the ArangoDB API doc . Examples iex&gt; collection ( &quot;users&quot; ) % ArangoXEcto.Migration.Collection { name : &quot;users&quot; , type : 2 ) iex&gt; collection ( &quot;users&quot; , :edge ) % ArangoXEcto.Migration.Collection { name : &quot;users&quot; , type : 3 ) iex&gt; collection ( &quot;users&quot; , :document , keyOptions : %{ type : :uuid } ) % ArangoXEcto.Migration.Collection { name : &quot;users&quot; , type : 2 , keyOptions : %{ type : :uuid } )","ref":"ArangoXEcto.Migration.html#collection/3","title":"ArangoXEcto.Migration.collection/3","type":"function"},{"doc":"Creates an object Will create the passed object, one of a collection, an index, analyzers or a view. Since view schemas and analyzers are just module definitions we can use them directly here. Therefore the module is just passed directly. There is the ability to pass additional options as the second argument Options :repo - The repo or connection to use for the migration create action :prefix - The prefix to use for tenant creation Examples Create a collection iex&gt; create ( collection ( &quot;users&quot; ) ) :ok Create an index iex&gt; create ( index ( &quot;users&quot; , [ :email ] ) :ok Create a view iex&gt; create ( MyProject.UsersView ) :ok","ref":"ArangoXEcto.Migration.html#create/2","title":"ArangoXEcto.Migration.create/2","type":"function"},{"doc":"Deletes an object Will delete an object passed, can only be a collection, indexes cannot be deleted here. This is because indexes have a randomly generated id and this needs to be known to delete the index, for now this is outside the scope of this project. Example iex&gt; drop ( collection ( &quot;users&quot; ) ) :ok","ref":"ArangoXEcto.Migration.html#drop/2","title":"ArangoXEcto.Migration.drop/2","type":"function"},{"doc":"Creates an edge collection struct Same as passing :edge as the second parameter to collection/3 .","ref":"ArangoXEcto.Migration.html#edge/2","title":"ArangoXEcto.Migration.edge/2","type":"function"},{"doc":"Creates an index struct Default index type is a hash. To change this pass the :type option in options. Options Options only apply to the creation of indexes and has no effect when using the drop/1 function. :type - The type of index to create Accepts: :fulltext , :geo , :hash , :persistent , :skiplist or :ttl :unique - If the index should be unique, defaults to false (hash, persistent &amp; skiplist only) :sparse - If index should be spares, defaults to false (hash, persistent &amp; skiplist only) :deduplication - If duplication of array values should be turned off, defaults to true (hash &amp; skiplist only) :minLength - Minimum character length of words to index (fulltext only) :geoJson - If a geo-spatial index on a location is constructed and geoJson is true, then the order within the array is longitude followed by latitude (geo only) :expireAfter - Time in seconds after a document's creation it should count as expired (ttl only) :name - The name of the index (usefull for phoenix constraints) Examples Create index on email field iex&gt; index ( &quot;users&quot; , [ :email ] ) % ArangoXEcto.Migration.Index { collection_name : &quot;users&quot; , fields : [ :email ] } Create dual index on email and ph_number fields iex&gt; index ( &quot;users&quot; , [ :email , :ph_number ] ) % ArangoXEcto.Migration.Index { collection_name : &quot;users&quot; , fields : [ :email , :ph_number ] } Create unique email index iex&gt; index ( &quot;users&quot; , [ :email ] , unique : true ) % ArangoXEcto.Migration.Index { collection_name : &quot;users&quot; , fields : [ :email ] , unique : true }","ref":"ArangoXEcto.Migration.html#index/3","title":"ArangoXEcto.Migration.index/3","type":"function"},{"doc":"Represent a collection in ArangoDB The attributes in this struct are directly passed to the ArangoDB API for creation. No validation is done on the attributes and is left to the database to manage.","ref":"ArangoXEcto.Migration.Collection.html","title":"ArangoXEcto.Migration.Collection","type":"module"},{"doc":"Creates a new Collection struct","ref":"ArangoXEcto.Migration.Collection.html#new/3","title":"ArangoXEcto.Migration.Collection.new/3","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.Collection.html#t:collection_option/0","title":"ArangoXEcto.Migration.Collection.collection_option/0","type":"type"},{"doc":"","ref":"ArangoXEcto.Migration.Collection.html#t:t/0","title":"ArangoXEcto.Migration.Collection.t/0","type":"type"},{"doc":"Represents a collection index in ArangoDB The attributes in this struct are directly passed to the ArangoDB API for creation. No validation is done on the attributes and is left to the database to manage.","ref":"ArangoXEcto.Migration.Index.html","title":"ArangoXEcto.Migration.Index","type":"module"},{"doc":"Creates a new Index struct","ref":"ArangoXEcto.Migration.Index.html#new/3","title":"ArangoXEcto.Migration.Index.new/3","type":"function"},{"doc":"","ref":"ArangoXEcto.Migration.Index.html#t:index_option/0","title":"ArangoXEcto.Migration.Index.index_option/0","type":"type"},{"doc":"","ref":"ArangoXEcto.Migration.Index.html#t:t/0","title":"ArangoXEcto.Migration.Index.t/0","type":"type"},{"doc":"Converts Ecto.Query structs into AQL syntax. This module is copied from https://github.com/ArangoDB-Community/arangodb_ecto/blob/master/lib/arangodb_ecto/query.ex . All credit goes to mpoeter , the original author. Please go check out the original of this file. This is an updated version for Ecto V3","ref":"ArangoXEcto.Query.html","title":"ArangoXEcto.Query","type":"module"},{"doc":"Creates an AQL query to fetch all entries from the data store matching the given Ecto query.","ref":"ArangoXEcto.Query.html#all/1","title":"ArangoXEcto.Query.all/1","type":"function"},{"doc":"Creates an AQL query to delete all entries from the data store matching the given Ecto query.","ref":"ArangoXEcto.Query.html#delete_all/1","title":"ArangoXEcto.Query.delete_all/1","type":"function"},{"doc":"A OR search query expression. Extention to the Ecto.Query api for arango searches. This function is the same as ArangoXEcto.Query.search/3 except implements as an or clause. This also follows the same syntax as the Ecto.Query.or_where/3 function.","ref":"ArangoXEcto.Query.html#or_search/3","title":"ArangoXEcto.Query.or_search/3","type":"macro"},{"doc":"An AND search query expression. Extention to the Ecto.Query api for arango searches. The expression syntax is exactly the same as the regular Ecto.Query.where/3 clause. Refer to that for more info on syntax or for advanced AQL queries see the section below. You will need to import this function like you do for Ecto.Query . Implementation This will store the search in the Ecto query where clause with a custom search operation. This is to prevent having to create a seperate query type. When converting the Ecto query to an AQL query, this is caught and changed into an AQL SEARCH expression. Using Analyzers and advanced AQL To save having to learn some new kind of query format for so many different possible search scenarios, you can just use AQL directly in. This is powered by the Ecto.Query.API.fragment/1 function. Below is an example of how you can implement using a custom analyzer: from ( UsersView ) |&gt; search ( [ uv ] , fragment ( &quot;ANALYZER(? == ?, &quot; identity &quot;)&quot; , uv . first_name , &quot;John&quot; ) ) |&gt; Repo . all ( )","ref":"ArangoXEcto.Query.html#search/3","title":"ArangoXEcto.Query.search/3","type":"macro"},{"doc":"Creates an AQL query to update all entries from the data store matching the given Ecto query.","ref":"ArangoXEcto.Query.html#update_all/1","title":"ArangoXEcto.Query.update_all/1","type":"function"},{"doc":"This module is a helper to automatically specify the primary key. The primary key is the Arango _key field but the _id field is also provided. Schema modules should use this module by add use ArangoXEcto.Schema to the module. The only exception to this is if the collection is an edge collection, in that case refer to ArangoXEcto.Edge. Example defmodule MyProject.Accounts.User do use ArangoXEcto.Schema import Ecto.Changeset schema &quot;users&quot; do field :first_name , :string field :last_name , :string timestamps ( ) end @doc false def changeset ( app , attrs ) do app |&gt; cast ( attrs , [ :first_name , :last_name ] ) |&gt; validate_required ( [ :first_name , :last_name ] ) end end","ref":"ArangoXEcto.Schema.html","title":"ArangoXEcto.Schema","type":"module"},{"doc":"Creates a foreign key with correct format Appends _id to the atom","ref":"ArangoXEcto.Schema.html#build_foreign_key/1","title":"ArangoXEcto.Schema.build_foreign_key/1","type":"function"},{"doc":"Defines an incoming relationship Behind the scenes this defines a many to many relationship so that Ecto can load the relationship using the built in functions. The use of this function MUST be accompanied by a outgoing/3 definition in the other target node. This will also define the __id__ field if it is not already defined so that ecto can map the relationship. Example defmodule MyProject.Post do use ArangoXEcto.Schema schema &quot;posts&quot; do field :title , :string # Will use the automatically generated edge incoming :users , MyProject.User # Will use the UserPosts edge incoming :users , MyProject.User , edge : MyProject.UserPosts end end","ref":"ArangoXEcto.Schema.html#incoming/3","title":"ArangoXEcto.Schema.incoming/3","type":"macro"},{"doc":"Defines indexs for collection dynamic creation The available options can be found in the ArangoXEcto.Migration.Index module. Example To create a generic hash index you don't need to pass the type. indexes [ [ fields : [ :email , :username ] ] ] To create a geoJson index set the type to geo and set geoJson to true. indexes [ [ fields : [ :point ] , type : :geo , geoJson : true ] ] To create a two seperate indexes just supply them seperately. indexes [ [ fields : [ :email ] ] , [ fields : [ :username ] ] ]","ref":"ArangoXEcto.Schema.html#indexes/1","title":"ArangoXEcto.Schema.indexes/1","type":"macro"},{"doc":"Defines an incoming relationship of one object Unlike incoming/3 , this does not create a graph relation and instead places the _id in a field. If the value passed to the name attribute is :user then the foreign key created on this schema will be :user_id and will store the full _id of that user. By storing the full _id , you are still able to perform full AQL queries. This MUST be accompanied by a one_outgoing/3 definition in the other target schema. Behind the scenes this injects the __id__ field to store the _id value and uses the built-in Ecto belongs_to/3 function. Options passed to the opts attribute are passed to the belongs_to/3 definition. Refrain from overriding the :references and :foreign_key attributes unless you know what you are doing. Example defmodule MyProject.Post do use ArangoXEcto.Schema schema &quot;posts&quot; do field :title , :string one_incoming :user , MyProject.User end end","ref":"ArangoXEcto.Schema.html#one_incoming/3","title":"ArangoXEcto.Schema.one_incoming/3","type":"macro"},{"doc":"Defines an outgoing relationship of one object Unlike outgoing/3 , this does not create a graph relation and instead places the _id in a field in the incoming schema. This MUST be accompanied by a one_incoming/3 definition in the other target schema. Behind the scenes this injects the __id__ field to store the _id value and uses the built-in Ecto has_one/3 function. Options passed to the opts attribute are passed to the has_many/3 definition. Refrain from overriding the :references and :foreign_key attributes unless you know what you are doing. Example defmodule MyProject.User do use ArangoXEcto.Schema schema &quot;users&quot; do field :name , :string one_outgoing :best_post , MyProject.Post end end","ref":"ArangoXEcto.Schema.html#one_outgoing/3","title":"ArangoXEcto.Schema.one_outgoing/3","type":"macro"},{"doc":"Defines collection options for dynamic creation When using dynamic mode for collection creation you may want to add options for when the collection is created. To add options that will be applied you pass them to as a keyword list. The available options can be found in the ArangoXEcto.Migration.Collection module. These options also work in edge collections. Example To specify a UUID as the collection key type you just supply it as the key option type. options [ keyOptions : %{ type : :uuid } ]","ref":"ArangoXEcto.Schema.html#options/1","title":"ArangoXEcto.Schema.options/1","type":"macro"},{"doc":"Defines an outgoing relationship of many objects Behind the scenes this defines a many to many relationship so that Ecto can load the relationship using the built in functions. The use of this function MUST be accompanied by a incoming/3 definition in the other target node. This will also define the __id__ field if it is not already defined so that ecto can map the relationship. Example defmodule MyProject.User do use ArangoXEcto.Schema schema &quot;users&quot; do field :name , :string # Will use the automatically generated edge outgoing :posts , MyProject.Post # Will use the UserPosts edge outgoing :posts , MyProject.Post , edge : MyProject.UserPosts end end","ref":"ArangoXEcto.Schema.html#outgoing/3","title":"ArangoXEcto.Schema.outgoing/3","type":"macro"},{"doc":"A wrapper around the Ecto.Schema.schema/2 to add the _id field","ref":"ArangoXEcto.Schema.html#schema/2","title":"ArangoXEcto.Schema.schema/2","type":"macro"},{"doc":"Defines a GeoJSON type for use in ecto schemas This will handle conversion of a Geo struct into a map for storage in the database. Use as the type of a field in an Ecto schema. For example: schema &quot;users&quot; do field :location , ArangoXEcto.Types.GeoJSON end","ref":"ArangoXEcto.Types.GeoJSON.html","title":"ArangoXEcto.Types.GeoJSON","type":"module"},{"doc":"Callback implementation for Ecto.Type.embed_as/1 .","ref":"ArangoXEcto.Types.GeoJSON.html#embed_as/1","title":"ArangoXEcto.Types.GeoJSON.embed_as/1","type":"function"},{"doc":"Callback implementation for Ecto.Type.equal?/2 .","ref":"ArangoXEcto.Types.GeoJSON.html#equal?/2","title":"ArangoXEcto.Types.GeoJSON.equal?/2","type":"function"},{"doc":"Helper methods","ref":"ArangoXEcto.Utils.html","title":"ArangoXEcto.Utils","type":"module"},{"doc":"Get system database connection","ref":"ArangoXEcto.Utils.html#get_system_db/1","title":"ArangoXEcto.Utils.get_system_db/1","type":"function"},{"doc":"ArangoSearch view module Defines a schema for a view to help create and manage Arango Search views. Creation Like for collections, the same static vs dynamic system is in place. If in dynamic mode and a collection that a view links to doesn't exist when querying, it will be created automatically. If in static mode an error will be raised if the the appropriate migration has not been made. Check out the ArangoXEcto.Migration module for more info. Like for collections, there is no check if a view exists when use the ArangoXEcto.aql_query/4 function and will error, so make sure the view exists first. Example defmodule Something.UserSearch do use ArangoXEcto.View alias ArangoXEcto.View.Link view &quot;user_search&quot; do primary_sort :created_at , :desc primary_sort :name store_value [ :email ] , :lz4 store_value [ :first_name , :last_name ] , :none link MyApp.Users , % Link { includeAllFields : true , fields : %{ name : % Link { analyzers : [ :text_en ] } } } options [ primarySortCompression : :lz4 ] end end Defining the analyzer module When in dynamic mode only the analyzer module needs to be passed to the view so that the analyzers will be automatically created. This option will be ignored in static mode. You can pass the analyzer module by passing the module to the :analyzer_module option on the use statement. use ArangoXEcto.View , analyzer_module : Something.Analyzers Querying Views essentially operate as virtual wrappers of Ecto Schemas. You can use them just like other schemas to query on and all the fields available are a culmination of the fields on the link schemas. This is how it works in the ArangoDB so to keep it true to function, it functions the same here. Querying is done exactly the same as a normal schema except the result will not be a struct. This is explained further below. iex&gt; Repo . all ( MyApp.UsersView ) [ %{ first_name : &quot;John&quot; , last_name : &quot;Smith&quot; } , _ ] Ecto Query Search Since the Arango Search function heavily relies on the AQL SEARCH operation it only makes sense for this to also work in Ecto Queries. You can find more info about this under ArangoXEcto.Query.search/3 and ArangoXEcto.Query.or_search/3 . The search operation functions the same as the Ecto.Query.where/3 clause so that can be used for more reference. You can also use fragments to use special analyzers, an example is given below. Note You must import the ArangoXEcto.Query function to use the search query macro. iex&gt; MyApp.UsersView |&gt; search ( gender : :male ) |&gt; Repo . all ( ) [ %{ gender : :male } , ... ] iex&gt; from ( MyApp.UsersView ) |&gt; search ( [ uv ] , uv . gender == :female ) |&gt; Repo . all ( ) [ %{ gender : :female } , ... ] iex&gt; UsersView |&gt; search ( [ uv ] , fragment ( &quot;ANALYZER(? == ?, &quot; identity &quot;)&quot; , uv . first_name , &quot;John&quot; ) ) |&gt; Repo . all ( ) [ %{ first_name : &quot;John&quot; } ] Unfortunately this won't work as an argument to the Ecto Query from macro (e.g. below) and will only work as above. # DO NOT USE THIS, IT WILL NOT WORK iex&gt; from ( uv in MyApp.UsersView , search : uv . first_name == &quot;John&quot; ) ArgumentError Of course you can also use AQL queries. Noting that you need to ensure the view is created already because it will not be checked. iex&gt; ArangoXEcto.aql query(Repo, ...&gt; &quot;FOR uv IN @@view SEARCH ANALYZER(uv.first name == @first_name, &quot;identity&quot;) RETURN uv&quot;, ...&gt; &quot;@view&quot;: UsersView.__view (:name), ...&gt; first_name: &quot;John&quot; ...&gt; ) {:ok, [%{&quot;first_name&quot; =&gt; &quot;John&quot;}]} Sorting Since views work with ecto queries as per usual, you can use functions such as sort in Ecto queries. If you want to sort based on the BM25 score for example, you can use fragments like below. iex&gt; UsersView ...&gt; |&gt; search ( gender : :male ) ...&gt; |&gt; order_by ( [ uv ] , fragment ( &quot;BM25(?)&quot; , uv ) ) ...&gt; |&gt; select ( [ uv ] , { uv . first_name , fragment ( &quot;BM25(?)&quot; , uv ) } ) [ { &quot;John&quot; , 1.8 } , { &quot;Bob&quot; , 1.8 } ] Loading results Due to how views function and the possibility of multiple types of returns, the result of a query cannot be automatically loaded to a struct with Ecto so you will have to use the ArangoXEcto.load/2 function. It does still load the values using the field loaders to the correct type, so if you don't need to load it into a struct you can skip this. iex&gt; Repo . all ( MyApp.UsersView ) |&gt; ArangoXEcto . load ( MyApp.User ) [ % User { first_name : &quot;John&quot; , last_name : &quot;Smith&quot; } , _ ] Since you can have multiple different schemas linked, the ArangoXEcto.load/2 function supports passing multiple module options that will match against the arango _id to load the correct module. iex&gt; Repo . all ( MyApp.UsersView ) |&gt; ArangoXEcto . load ( [ MyApp.User , MyApp.Post ] ) [ % User { first_name : &quot;John&quot; , last_name : &quot;Smith&quot; } , % Post { name : &quot;abc&quot; } ]","ref":"ArangoXEcto.View.html","title":"ArangoXEcto.View","type":"module"},{"doc":"Generates the view definition This takes the macros that define the view and converts it into a definition for use on creation.","ref":"ArangoXEcto.View.html#definition/1","title":"ArangoXEcto.View.definition/1","type":"function"},{"doc":"Defines a link field on the view. This can be supplied more than once in a view block to enable multiple link definitions. Each link can be of different documents or edges, this is sorted on querying as mentioned at the top of this module. Links are defined by the ArangoXEcto.View.Link module. Parameters * ` :schema ` - the module of the document or edge the link should be created on * ` :link ` - the link definition , see ` ArangoXEcto.View.Link `","ref":"ArangoXEcto.View.html#link/2","title":"ArangoXEcto.View.link/2","type":"macro"},{"doc":"Defines options for view creation. Options The available options are directly from the ArangoDB View definition . * ` :consolidationIntervalMsec ` - wait at least this many milliseconds between applying consolidationPolicy to consolidate View data store and possibly release space on the filesystem . * ` :consolidationPolicy ` - the consolidation policy to apply for selecting data store segment merge candidates . * ` :commitIntervalMsec ` - wait at least this many milliseconds between committing View data store changes and making documents visible to queries . * ` :writebufferSizeMax ` - maximum memory byte size per writer ( segment ) before a writer ( segment ) flush is triggered . * ` :writebufferIdle ` - maximum number of writers ( segments ) cached in the pool . * ` :writebufferActive ` - maximum number of concurrent active writers ( segments ) that perform a transaction . Other writers ( segments ) wait till current active writers ( segments ) finish . * ` :cleanupIntervalStep ` - waits at least this many commits between removing unused files in its data directory . * ` :primarySortCompression ` - defines how to compress the primary sort data ( introduced in ArangoDB v3 . 7.0 ) . See ` t :ArangoXEcto . View . compression / 0 ` . ` :lz4 ` ( default ) - use LZ4 fast compression ` :none ` - disable compression to trade space for speed","ref":"ArangoXEcto.View.html#options/1","title":"ArangoXEcto.View.options/1","type":"macro"},{"doc":"Defines a primary sort field on the view. Only a field name is required but a direction can also be supplied. By default the direction will be ascending (:asc). This can be supplied more than once in a view block to enable multiple primary sorts. Parameters * ` :field ` - atom name of the field to be used as a primary sort * ` :direction ` - either ` :asc ` or ` :desc ` for sort direction","ref":"ArangoXEcto.View.html#primary_sort/2","title":"ArangoXEcto.View.primary_sort/2","type":"macro"},{"doc":"Defines a stored value on the view. Only the fields used is required but the compression method can also be supplied. By default the compression will be :lz4. This can be supplied more than once in a view block to enable multiple stored value definitions. Parameters * ` :fields ` - a list of atoms for fields for the stored value * ` :compression ` - either ` :none ` or ` :lz4 ` for compression , see ` t :ArangoXEcto . View . compression / 0 `","ref":"ArangoXEcto.View.html#store_value/2","title":"ArangoXEcto.View.store_value/2","type":"macro"},{"doc":"Defines a view with a view name and defenitions. Can only be defined once per view module.","ref":"ArangoXEcto.View.html#view/2","title":"ArangoXEcto.View.view/2","type":"macro"},{"doc":"","ref":"ArangoXEcto.View.html#t:compression/0","title":"ArangoXEcto.View.compression/0","type":"type"},{"doc":"","ref":"ArangoXEcto.View.html#t:name/0","title":"ArangoXEcto.View.name/0","type":"type"},{"doc":"Defines the structure of a link This follows the format of the link properties defined in ArangoDB. For more information check out the ArangoDB docs . Example %ArangoXEcto.View.Link{ includeAllFields : true , fields : %{ name : % ArangoXEcto.View.Link { analyzers : [ :text_en ] } } }","ref":"ArangoXEcto.View.Link.html","title":"ArangoXEcto.View.Link","type":"module"},{"doc":"Converts to a map for api call This will convert any structs into maps in a deeply nested link definition.","ref":"ArangoXEcto.View.Link.html#to_map/1","title":"ArangoXEcto.View.Link.to_map/1","type":"function"},{"doc":"Validates a link This checks the types and strucute of a provided link struct. If a link is invalid for any reason this will return false. Using this allows for validation before sending to the DB.","ref":"ArangoXEcto.View.Link.html#valid?/1","title":"ArangoXEcto.View.Link.valid?/1","type":"function"},{"doc":"","ref":"ArangoXEcto.View.Link.html#t:t/0","title":"ArangoXEcto.View.Link.t/0","type":"type"},{"doc":"Migration task helper functions","ref":"Mix.ArangoXEcto.html","title":"Mix.ArangoXEcto","type":"module"},{"doc":"Creates the database specified in the config","ref":"Mix.ArangoXEcto.html#create_base_database/0","title":"Mix.ArangoXEcto.create_base_database/0","type":"function"},{"doc":"Creates a new database of name","ref":"Mix.ArangoXEcto.html#create_database/1","title":"Mix.ArangoXEcto.create_database/1","type":"function"},{"doc":"Creates a document to store migrations Creates a document with key of the database name. Seperating the migrations into seperate documents allow for more organisation and better debugging.","ref":"Mix.ArangoXEcto.html#create_migration_document/0","title":"Mix.ArangoXEcto.create_migration_document/0","type":"function"},{"doc":"Creates the migrations collection Creates an empty system collection to store migrations.","ref":"Mix.ArangoXEcto.html#create_migrations/0","title":"Mix.ArangoXEcto.create_migrations/0","type":"function"},{"doc":"Gets the default repo's database name","ref":"Mix.ArangoXEcto.html#get_database_name!/0","title":"Mix.ArangoXEcto.get_database_name!/0","type":"function"},{"doc":"Gets the default repo The first in the list of running repos is used.","ref":"Mix.ArangoXEcto.html#get_default_repo!/0","title":"Mix.ArangoXEcto.get_default_repo!/0","type":"function"},{"doc":"Gets all the migrated versions","ref":"Mix.ArangoXEcto.html#migrated_versions/1","title":"Mix.ArangoXEcto.migrated_versions/1","type":"function"},{"doc":"Gets the path to the priv repo folder Will return a full file path to the priv/repo folder.","ref":"Mix.ArangoXEcto.html#path_to_priv_repo/1","title":"Mix.ArangoXEcto.path_to_priv_repo/1","type":"function"},{"doc":"Removes a version from the migrations collection","ref":"Mix.ArangoXEcto.html#remove_version/2","title":"Mix.ArangoXEcto.remove_version/2","type":"function"},{"doc":"Creates a timestamp for the migration file","ref":"Mix.ArangoXEcto.html#timestamp/0","title":"Mix.ArangoXEcto.timestamp/0","type":"function"},{"doc":"Updates the migrated versions in the migration collection","ref":"Mix.ArangoXEcto.html#update_versions/2","title":"Mix.ArangoXEcto.update_versions/2","type":"function"},{"doc":"Specifies a migration version type","ref":"Mix.ArangoXEcto.html#t:version/0","title":"Mix.ArangoXEcto.version/0","type":"type"},{"doc":"Generates a migration. The repository must be set under :ecto_repos in the current app configuration or given via the -r option. Examples mix ecto . gen . migration create_users mix ecto . gen . migration create_users - r Custom.Repo The generated migration filename will be prefixed with the current timestamp in UTC which is used for versioning and ordering. By default, the migration will be generated to the priv/YOUR_REPO/migrations directory of the current application but it can be configured to be any subdirectory of priv by specifying the :priv key under the repository configuration. This generator will automatically open the generated file if you have ECTO_EDITOR set in your environment variable. Command line options -r , --repo - the repo to generate migration for","ref":"Mix.Tasks.Ecto.Gen.Migration.Arango.html","title":"Mix.Tasks.Ecto.Gen.Migration.Arango","type":"task"},{"doc":"Runs Migration/Rollback functions from migration modules","ref":"Mix.Tasks.Ecto.Migrate.Arango.html","title":"Mix.Tasks.Ecto.Migrate.Arango","type":"task"},{"doc":"","ref":"Mix.Tasks.Ecto.Migrate.Arango.html#down/1","title":"Mix.Tasks.Ecto.Migrate.Arango.down/1","type":"function"},{"doc":"Sets up all necessary aspects of the adapter Adds migrations collection in _systems db for migrations and creates the database","ref":"Mix.Tasks.Ecto.Setup.Arango.html","title":"Mix.Tasks.Ecto.Setup.Arango","type":"task"},{"doc":"ArangoXEcto is an all-in-one Arango database adapter for the Elixir Ecto package. It has full support for Graphing , Arango Search , Geo Functions , AQL Integration , amongst other features.","ref":"readme.html","title":"ArangoX Ecto","type":"extras"},{"doc":"ArangoX Ecto Table of Contents About The Project Built With Getting Started Prerequisites Installation Usage Static or Dynamic Basic Usage Repo Setup Schema Setup Migration Setup Raw AQL queries Graph Relations Generation of Edge Modules Graphing in Ecto One-to-One relationships How it works Example More information Arango Search Further Usage Roadmap Contributing License Contact Acknowledgements","ref":"readme.html#table-of-contents","title":"ArangoX Ecto - Table of Contents","type":"extras"},{"doc":"After playing around with different packages that implemented ArangoDB in Elixir, I found that there wasn't a package that suited my needs. I needed ArangoDB to work with Ecto seamlessly but there was no up-to-date adapter for Ecto available. ArangoX Ecto uses the power of ArangoX to communicate with ArangoDB and Ecto for the API in Elixir. Ecto is integrated with many other packages and can now be used with ArangoDB thanks to this package. From version 1.0.0 onward graph relationships work seamlessly in Ecto. Built With Arangox","ref":"readme.html#about-the-project","title":"ArangoX Ecto - About The Project","type":"extras"},{"doc":"To get the adapter integrated with your project, follow these simple steps. Prerequisites Elixir 1.12.3+ / Erlang OTP 22.2+ (Others may versions may work but this is the oldest it is tested on) Installation Add the following line to your mix dependencies to get started. { :arangox_ecto , &quot;~&gt; 1.3&quot; }","ref":"readme.html#getting-started","title":"ArangoX Ecto - Getting Started","type":"extras"},{"doc":"To connect to the database you need to specify the config values like so: config :my_app , MyApp.Repo , database : &quot;my_db&quot; , endpoints : &quot;http://1.2.3.4:8529&quot; Only database and endpoints are required but there are other available options can be found in the Arangox docs . The database should be setup using $ mix ecto.setup.arango Static or Dynamic In addition to default config variables, the static boolean config option can be passed to force the use of migrations. By default the value is false and collections that don't exist will be created on insert and on query no error will be raised. If set to true , any collections that do not exist on insert or query will result in an error to be raised. Whether dynamic (default) or static is chosen defends on the database design of the project. For a production setup where lots of control is required, it is recommended to have static set to true . If using static in production is better why is dynamic the default? Dynamic mode is the default because it is easier for development and testing then after that point static mode can be turned on for production. Basic Usage Repo Setup To use the adapter in your repo, make sure your repo uses the ArangoXEcto.Adapter module for the adapter. defmodule MyApp.Repo do use Ecto.Repo , otp_app : :my_app , adapter : ArangoXEcto.Adapter end Schema Setup Since ArangoDB uses a slightly different id system, your schema must use the ArangoXEcto.Schema instead of Ecto.Schema . defmodule MyApp.Accounts.User do use ArangoXEcto.Schema import Ecto.Changeset schema &quot;users&quot; do field :first_name , :string field :last_name , :string timestamps ( ) end @doc false def changeset ( app , attrs ) do app |&gt; cast ( attrs , [ :first_name , :last_name ] ) |&gt; validate_required ( [ :first_name , :last_name ] ) end end The options/1 and indexes/1 options can be optionally called inside the schema to set options and indexes to be created in dynamic mode. These options will have no effect in static mode. For example, if you wanted to use a uuid as the key type and create an index on the email the following can be used. defmodule MyApp.Accounts.User do use ArangoXEcto.Schema import Ecto.Changeset options [ keyOptions : %{ type : :uuid } ] indexes [ [ fields : [ :email ] ] ] schema &quot;users&quot; do field :email , :string timestamps ( ) end @doc false def changeset ( app , attrs ) do app |&gt; cast ( attrs , [ :first_name , :last_name ] ) |&gt; validate_required ( [ :first_name , :last_name ] ) end end Please refer to the Schema documentation for more information on the available options. Migration Setup Using migrations is only required in static mode If in dynamic mode (default), the adapter will automatically create collections if they don't already exist but there are cases where you might need to use a static system and hence migrations are required. For example, if you needed to create indexes as well, the following would be used. Refer to ArangoXEcto.Migration for more information on usage. defmodule MyApp.Repo.Migrations.CreateUsers do use ArangoXEcto.Migration def up do create ( collection ( :users ) ) create ( index ( &quot;users&quot; , [ :email ] ) ) end def down do drop ( collection ( :users ) ) end end Raw AQL queries A lot of the time it is far more efficient to just run a raw AQL query, there's a function for that. ArangoXEcto . aql_query ( Repo , &quot;FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var&quot; , fname : &quot;John&quot; , lname : &quot;Smith&quot; ) This query will return a result such as: { :ok , [ %{ &quot;_id&quot; =&gt; &quot;users/12345&quot; , &quot;_key&quot; =&gt; &quot;12345&quot; , &quot;_rev&quot; =&gt; &quot;_bHZ8PAK---&quot; , &quot;first_name&quot; =&gt; &quot;John&quot; , &quot;last_name&quot; =&gt; &quot;Smith&quot; } ] } This is awesome functionality, but a lot of the time we will want to resemble a specific struct. This is actually quite easy with the help of the ArangoXEcto.load/2 function. The same query above could be extended to also convert the output: ArangoXEcto . aql_query ( Repo , &quot;FOR var in users FILTER var.first_name == @fname AND var.last_name == @lname RETURN var&quot; , fname : &quot;John&quot; , lname : &quot;Smith&quot; ) |&gt; case do { :ok , results } -&gt; results |&gt; ArangoXEcto . load ( User ) { :error , _reason } -&gt; [ ] end This will return something like: [ % User { id : &quot;12345&quot; , first_name : &quot;John&quot; , last_name : &quot;Smith&quot; } ] This is clearly a much better representation of the result and can be used in further Ecto methods. Graph Relations The adapter will dynamically create and manage edge collections. Each edge collection will be created as an Ecto schema when they are first used. This will allow for more extensibility through Ecto onto the edges. Generation of Edge Modules The module will be created under the closest common parent module of the passed modules plus the Edges alias. The order of the edge name will always be alphabetical to prevent duplicate edges. For example, if the modules were MyApp.Apple.User and MyApp.Apple.Banana.Post then the edge would be created at MyApp.Apple.Edges.PostsUsers . This assumes that the edge collection name was generated and not explicitly defined, if it was PostsUsers would be replaced with the camel case of that collection name (i.e. posts_users ). Graphing in Ecto From version 1.0.0 onwards, edges are represented as traditional relationships in Ecto. This approach allows for the simplicity of simple management of edges but the complex querying through AQL. The relationships are represented by an outgoing/3 and incoming/3 in the respective schemas. For example if you wanted to create edges between A and B, with the direction A -&gt; B then A would have outgoing/3 and B would have incoming/3 . This directionality is simply for what is put in the _from and _to fields in the Arango edge collection, if you don't care about directionality, you don't have to worry as much about which schema has outgoing/3 and incoming/3 . One-to-One relationships Additionally, one_outgoing/3 and one_incoming/3 can be used for one-to-one relationships. These do not actually create edges but just store the _id of the target in a field instead. The order of the outgoing &amp; incoming in schemas does matter as a field will be created in the incoming schema. One-to-Many relationships Graph relationships are a good substitute for many-to-many relationships in a traditional relational database. One-to-Many relationships in ArangoDB will function exactly the same as in a relational database. Therefore you can just use the regular Ecto.Schema.has_many/3 and Echo.Schema.belongs_to/3 functions as documented in the Ecto docs . How it works Behind the scenes these outgoing &amp; incoming helper macros are simply wrappers around the Ecto function Ecto.Schema.many_to_many/3 . The one-to-one outgoing &amp; incoming macros are just wrappers around Ecto.Schema.has_one/3 and Ecto.Schema.belongs_to/3 respectively. Hence once setup the regular methods for handling Ecto relationships can be used. In order to delete a specific edge, you can do it exactly as you would any other Ecto struct (since after all it is one) or Ecto relation. Querying of edges can be done either through using an AQL query or by using Ecto methods. Example Let's say you wanted to create an edge between your Post and User schema. You could implement it as follows: defmodule MyProject.User do use ArangoXEcto.Schema schema &quot;users&quot; do field :name , :string # Will use the automatically generated edge outgoing :posts , MyProject.Post # Or use the UserPosts edge # outgoing :posts, MyProject.Post, edge: MyProject.UserPosts end end defmodule MyProject.Post do use ArangoXEcto.Schema schema &quot;posts&quot; do field :title , :string # Will use the automatically generated edge incoming :users , MyProject.User # Or use the UserPosts edge # incoming :users, MyProject.User, edge: MyProject.UserPosts end end More information To learn about using the schema functions for representing graph relationships and examples, read the docs at ArangoXEcto.Schema . To read more about Edge Schemas and how to extend edge schemas to add additional fields, read the docs on ArangoXEcto.Edge . To learn how to use the helper functions (as well as other useful methods) check out the full documentation . Arango Search As of version 1.3.0 Arango Search functionality is built in. This builds off of the Ecto schema module for views so they can be searched and interacted with like regular collections. The view schemas are not exactly the same as collection or edge schemas because the views don't actually store or hold any data and aren't structs themselves. Views act as alias interfaces that over the collections and edge schemas. Querying Querying views works exactly the same as querying a regular collection or edge schema. For example, the following will work as you would expect. # Module definition defmodule UsersView do use ArangoXEcto.View alias ArangoXEcto.View.Link view &quot;user_search&quot; do primary_sort ( :created_at , :asc ) store_value ( [ :first_name ] , :lz4 ) link ( User , % Link { includeAllFields : true , fields : %{ last_name : % Link { analyzers : [ :text_en ] } } } ) options ( primarySortCompression : :lz4 ) end end # Querying iex&gt; Repo . all ( UsersView ) [ % User { first_name : &quot;John&quot; , last_name : &quot;Smith&quot; } , _ ] In addition to regular querying ArangoDB has the AQL SEARCH function for performing searches on views. This is implemented in ArangoXEcto using an additional Ecto query macro. The search/3 and or_search/3 macros are provided to utalise this functionality. Details of how to use these functions can be found in ArangoXEcto.Query and the ArangoXEcto.View modules. Behind the scenes the search/3 and or_search/3 are just wrappers around the Ecto where clause that gets unwrapped at the point when the AQL query is generated. Analyzers Analyzers can also be defined from within an analyzer module. Refer to the ArangoXEcto.Analyzer module for documentation. Further Usage For more examples and full documentation, please refer to the Documentation .","ref":"readme.html#usage","title":"ArangoX Ecto - Usage","type":"extras"},{"doc":"See the the roadmap for a list of proposed features (and known issues) planned.","ref":"readme.html#roadmap","title":"ArangoX Ecto - Roadmap","type":"extras"},{"doc":"Contributions are what make the open source community such an amazing place to be learn, inspire, and create. Any contributions you make are greatly appreciated . Checkout the Contributing Guide .","ref":"readme.html#contributing","title":"ArangoX Ecto - Contributing","type":"extras"},{"doc":"Distributed under the Apache 2.0 License. See LICENSE for more information.","ref":"readme.html#license","title":"ArangoX Ecto - License","type":"extras"},{"doc":"Project Home: https://github.com/TomGrozev/arangox_ecto","ref":"readme.html#contact","title":"ArangoX Ecto - Contact","type":"extras"},{"doc":"mpoeter - Wrote the original Ecto Query to AQL code kianmeng - README changes &amp; CI dep change bodbdigr - Fixed AQL query typespec hengestone - Added option to use different migrations directory","ref":"readme.html#acknowledgements","title":"ArangoX Ecto - Acknowledgements","type":"extras"}]